@@ -23,44 +23,58 @@ class WSA4000Properties(object):
     CAPTURE_FREQ_RANGES = [(0, 40*M, I_ONLY), (90*M, 10000*M, IQ)]
     SWEEP_FREQ_RANGE = (90*M, 10000*M)
 
-    FULL_BW = 125*M
-    USABLE_BW = 90*M
-    MIN_TUNABLE = 90*M
-    MAX_TUNABLE = 10000*M
-    MIN_DECIMATION = 4
-    MAX_DECIMATION = 1023
+    RFE_MODES = ('ZIF',)
+
+    FULL_BW = {'ZIF': 125*M}
+    USABLE_BW = {'ZIF': 90*M}
+    MIN_TUNABLE = {'ZIF': 90*M}
+    MAX_TUNABLE = {'ZIF': 10000*M}
+    MIN_DECIMATION = {'ZIF': 4}
+    MAX_DECIMATION = {'ZIF': 1023}
     DECIMATED_USABLE = 0.5
+    PASS_BAND_CENTER = {'ZIF': 0.5}
     DC_OFFSET_BW = 240000 # XXX: an educated guess
     TUNING_RESOLUTION = 100000
-
+    FSHIFT_AVAILABLE = {'ZIF': True}
     SWEEP_SETTINGS = ['fstart', 'fstop', 'fstep', 'fshift', 'decimation',
         'antenna', 'gain', 'ifgain', 'spp', 'ppb', 'dwell_s', 'dwell_us',
         'trigtype', 'level_fstart', 'level_fstop', 'level_amplitude']
 
 
-class WSA5000Properties(object):
-    model = 'WSA5000'
+class WSA5000_220Properties(object):
+    model = 'WSA5000-220'
+    MINIMUM_FW_VERSION = '3.2.0-rc1'
 
     ADC_DYNAMIC_RANGE = 72.5
     NOISEFLOOR_CALIBRATION = -10
     CAPTURE_FREQ_RANGES = [(0, 50*M, I_ONLY), (100*M, 20000*M, IQ)]
     SWEEP_FREQ_RANGE = (100*M, 20000*M)
 
-    FULL_BW = 125*M
-    USABLE_BW = 100*M
-    MIN_TUNABLE = 100*M
-    MAX_TUNABLE = 20000*M
-    MIN_DECIMATION = 2
-    MAX_DECIMATION = 1024
-    DECIMATED_USABLE = 0.875 # XXX: to be confirmed
+    RFE_MODES = ('ZIF', 'SH', 'HDR')
+
+    FULL_BW = {'ZIF': 125*M, 'HDR': 162760, 'SH': 62500000}
+    USABLE_BW = {'ZIF': 100*M, 'HDR': 100000, 'SH': 40000000}
+    MIN_TUNABLE = {'ZIF': 50*M, 'HDR': 50*M, 'SH': 50*M}
+    MAX_TUNABLE = {'ZIF': 20000*M, 'HDR': 20000*M, 'SH': 20000*M}
+    MIN_DECIMATION = {'ZIF': 4, 'HDR': None, 'SH': 4}
+    MAX_DECIMATION = {'ZIF': 1024, 'HDR': None, 'SH': 4}
+    DECIMATED_USABLE = 0.80
+    PASS_BAND_CENTER = {'ZIF': 0.5, 'HDR': 0.6, 'SH': 0.6}
     DC_OFFSET_BW = 240000 # XXX: an educated guess
     TUNING_RESOLUTION = 100000
-
+    FSHIFT_AVAILABLE = {'ZIF': True, 'HDR': False, 'SH': True}
+    MAX_FSHIFT = {'ZIF': 62.5*M}
     SWEEP_SETTINGS = ['fstart', 'fstop', 'fstep', 'fshift', 'decimation',
         'attenuator', 'ifgain', 'spp', 'ppb', 'dwell_s', 'dwell_us',
         'trigtype', 'level_fstart', 'level_fstop', 'level_amplitude']
 
 
+class WSA5000_208Properties(WSA5000_220Properties):
+    model = 'WSA5000-208'
+
+    MAX_TUNABLE = {'ZIF': 8000*M, 'HDR': 8000*M, 'SH': 8000*M}
+
+
 class WSA(object):
     """
     Interface for WSA4000 and WSA5000
@@ -89,6 +103,14 @@ class WSA(object):
         if not connector:
             connector = PlainSocketConnector()
         self.connector = connector
+        self._output_file = None
+
+    def set_capture_output(self, output_file=None):
+        """
+        Dump a recording of all the received packets to output_file
+        """
+        self.connector.set_recording_output(output_file)
+        self._output_file = output_file
 
     @sync_async
     def connect(self, host):
@@ -101,8 +123,12 @@ class WSA(object):
         device_id = (yield self.scpiget(":*idn?"))
         if device_id.startswith('ThinkRF,WSA4000'):
             self.properties = WSA4000Properties
+        elif device_id.startswith('ThinkRF,WSA5000-208'):
+            self.properties = WSA5000_208Properties
         else:
-            self.properties = WSA5000Properties
+            self.properties = WSA5000_220Properties
+
+        self.fw_version = device_id.split(',')[-1]
 
     def disconnect(self):
         """
@@ -161,6 +187,7 @@ class WSA(object):
             buf = yield self.scpiget(":INPUT:MODE?")
             mode = buf.strip()
         else:
+
             self.scpiset(":INPUT:MODE %s" % mode)
 
         yield mode
@@ -574,7 +601,7 @@ class WSA(object):
         """
         Remove all entries from the sweep list.
         """
-        self.scpiset(":sweep:entry:del all")
+        self.scpiset(":sweep:entry:delete all")
 
 
     def sweep_start(self, start_id = None):
@@ -643,7 +670,7 @@ class WSA(object):
             enable = yield self.scpiget(":INPUT:ATTENUATOR?")
             enable = bool(int(enable))
         else:
-            self.scpiset(":INPUT:ATTENUATOR %s" % 1 if enable else 0)
+            self.scpiset(":INPUT:ATTENUATOR %s" % (1 if enable else 0))
         yield enable
 
     @sync_async
@@ -682,6 +709,7 @@ class WSA(object):
             'ppb': self.ppb,
             'trigger': self.trigger,
             'attenuator': self.attenuator,
+            'rfe_mode': self.rfe_mode,
             }
 
         for k, v in settings.iteritems():
@@ -706,3 +734,4 @@ def parse_discovery_response(response):
 
 # for backwards compatibility
 WSA4000 = WSA
+
