@@ -13,6 +13,7 @@ class SweepStep(namedtuple('SweepStep', '''
         points
         bins_skip
         bins_run
+        bins_pass
         bins_keep
         ''')):
     """
@@ -25,11 +26,12 @@ class SweepStep(namedtuple('SweepStep', '''
     :param points: samples to capture
     :param bins_skip: number of FFT bins to skip from left
     :param bins_run: number of usable FFT bins each step
+    :param bins_pass: number of bins from first step to discard from left
     :param bins_keep: total number of bins to keep from all steps
     """
     __slots__ = []
 
-    def to_sweep_entry(self, device, **kwargs):
+    def to_sweep_entry(self, device, rfe_mode, **kwargs):
         """
         Create a SweepEntry for device matching this SweepStep,
 
@@ -39,66 +41,24 @@ class SweepStep(namedtuple('SweepStep', '''
         if self.points > 32*1024:
             raise SweepDeviceError('large captures not yet supported')
 
-        return SweepEntry(
+        s = SweepEntry(
             fstart=self.fcenter,
             fstop=min(self.fcenter + (self.steps + 0.5) * self.fstep,
-                device.properties.MAX_TUNABLE),
+                device.properties.MAX_TUNABLE[rfe_mode]),
             fstep=self.fstep,
             fshift=self.fshift,
             decimation=self.decimation,
             spp=self.points,
             ppb=1,
             **kwargs)
+        return s
 
     @property
     def steps(self):
-        return math.ceil(float(self.bins_keep) / self.bins_run)
+        return math.ceil(float(
+            self.bins_keep + self.bins_pass) / self.bins_run)
 
 
-    def trim(self, device, fstart, fstop):
-        """
-        Create a SweepStep that only includes steps within fstart to fstop
-        or return None if no part of this SweepStep overlaps
-        """
-        if fstop <= fstart:
-            return
-
-        steps = self.steps
-        bin_width = float(device.properties.FULL_BW
-            ) / self.decimation / self.points
-        start_centered = self.bins_skip - self.points / 2
-        start_off = start_centered * bin_width - self.fshift
-        stop_centered = start_centered + self.bins_run
-        stop_off = stop_centered * bin_width - self.fshift
-
-        start_step = math.floor((fstart - self.fcenter - start_off) / self.fstep)
-        stop_step = steps + math.ceil(
-            (fstop - self.fcenter - stop_off) / self.fstep)
-        if steps <= start_step or stop_step <= 0:
-            return
-
-        last_bins_run = self.bins_keep % self.bins_run
-        if steps - 1 == start_step and last_bins_run:
-            # starting from last step special case: fstart might be
-            # after the remaining samples
-            last_stop_centered = start_centered + last_bins_run
-            last_stop_off = last_stop_centered * bin_width - self.fshift
-            last_stop = self.fcenter + steps * self.fstep + last_stop_off
-            if last_stop <= fstart:
-                return
-
-        trim_left = max(0, start_step)
-        trim_right = steps - min(steps, stop_step)
-        return SweepStep(
-            fcenter=self.fcenter + trim_left * self.fstep,
-            fstep=self.fstep,
-            fshift=self.fshift,
-            decimation=self.decimation,
-            points=self.points,
-            bins_skip=self.bins_skip,
-            bins_run=self.bins_run,
-            bins_keep=self.bins_keep - (trim_left + trim_right) * self.bins_run,
-            )
 
 
 class SweepDeviceError(Exception):
@@ -129,9 +89,6 @@ class SweepDevice(object):
                 raise SweepDeviceError(
                     "async_callback not applicable for sync operation")
         self._prev_sweep_id = None
-        self._trigger_sweep = False
-        self._trigger_id = None
-        self._trigger_data = {}
         self.async_callback = async_callback
         self.continuous = False
         self.context_bytes_received = 0
@@ -145,8 +102,10 @@ class SweepDevice(object):
     connector = property(lambda self: self.real_device.connector)
 
     def capture_power_spectrum(self,
-            fstart, fstop, rbw, device_settings=None,
-            triggers=None, continuous=False,
+            fstart, fstop, rbw,
+            device_settings=None,
+            mode='ZIF',
+            continuous=False,
             min_points=128, max_points=8192):
         """
         Initiate a capture of power spectral density by
@@ -161,7 +120,8 @@ class SweepDevice(object):
         :type rbw: float
         :param device_settings: antenna, gain and other device settings
         :type dict:
-        :param triggers: list of :class:`TriggerSettings` instances or None
+        :param mode: sweep mode, 'ZIF left band' or 'ZIF'
+        :type mode: string
         :param continuous: async continue after first sweep
         :type continuous: bool
         :param min_points: smallest number of points per capture from real_device
@@ -169,10 +129,6 @@ class SweepDevice(object):
         :param max_points: largest number of points per capture from real_device
                            (due to decimation limits points returned may be larger)
         :type max_points: int
-
-        When triggers are provided nothing will be captured until one of the
-        triggers is satisfied. The trigger data received is combined with a full
-        sweep before being returned.
         """
         if continuous and not self.async_callback:
             raise SweepDeviceError(
@@ -185,49 +141,26 @@ class SweepDevice(object):
         self.real_device.request_read_perm()
 
         self.fstart, self.fstop, self.plan = plan_sweep(self.real_device,
-            fstart, fstop, rbw, min_points, max_points)
+            fstart, fstop, rbw, mode, min_points, max_points)
 
-        result = self._perform_trigger_sweep(triggers)
-        if result == 'async waiting':
-            return
+        self.sweep_segments = []
+        for ss in self.plan:
+            k = ss.bins_keep
+            seg = min(k, ss.bins_run - ss.bins_pass)
+            while True:
+                self.sweep_segments.append(seg)
+                k -= seg
+                if k <= 0:
+                    break
+                seg = min(ss.bins_run, k)
 
         return self._perform_full_sweep()
 
-
-    def _perform_trigger_sweep(self, triggers):
-        entries = []
-        if not triggers:
-            return
-        for t in triggers:
-            if t.trigtype != 'LEVEL':
-                raise SweepDeviceError('only level triggers supported')
-            tplan = trim_sweep_plan(self.real_device,
-                self.plan, t.fstart, t.fstop)
-            for ss in tplan:
-                entries.append(ss.to_sweep_entry(self.real_device,
-                    dwell_us=1,
-                    trigtype=t.trigtype,
-                    level_fstart=t.fstart,
-                    level_fstop=t.fstop,
-                    level_amplitude=t.amplitude,
-                    **self.device_settings))
-        if not entries:
-            return
-
-        if self.async_callback:
-            self.connector.vrt_callback = self._vrt_receive
-            self._start_sweep(entries, trigger=True)
-            return 'async waiting'
-        assert not self._trigger_data
-        self._start_sweep(entries, trigger=True)
-        while not self._trigger_data:
-            result = self._vrt_receive(self.real_device.read())
-
-
     def _perform_full_sweep(self):
         entries = []
         for ss in self.plan:
             entries.append(ss.to_sweep_entry(self.real_device,
+                'ZIF', # TODO: change when we support different rfe_modes
                 **self.device_settings))
 
         if self.async_callback:
@@ -246,7 +179,7 @@ class SweepDevice(object):
             result = self._vrt_receive(self.real_device.read())
         return result
 
-    def _start_sweep(self, entries, trigger=False):
+    def _start_sweep(self, entries):
         self.real_device.abort()
         self.real_device.flush()
         self.real_device.sweep_clear()
@@ -259,12 +192,8 @@ class SweepDevice(object):
         self._ss_index = 0
         self._ss_received = 0
         self.bins = []
-        self.real_device.sweep_iterations(
-            0 if trigger or self.continuous else 1)
+        self.real_device.sweep_iterations(0 if self.continuous else 1)
         self.real_device.sweep_start(self._sweep_id)
-        self._trigger_sweep = trigger
-        if trigger:
-            self._trigger_id = self._sweep_id
 
     def _vrt_receive(self, packet):
         packet_bytes = packet.size * 4
@@ -276,7 +205,7 @@ class SweepDevice(object):
 
         self.data_bytes_received += packet_bytes
         sweep_id = self._vrt_context.get('sweepid')
-        if sweep_id not in (self._sweep_id, self._trigger_id):
+        if sweep_id != self._sweep_id:
             if sweep_id == self._prev_sweep_id:
                 self.past_end_bytes_discarded += packet_bytes
             else:
@@ -289,25 +218,20 @@ class SweepDevice(object):
             "missing required context, sweep failed")
 
         freq = self._vrt_context['rffreq']
-        if self._trigger_sweep or sweep_id == self._trigger_id:
-            self._trigger_data[freq] = packet
-            if self._trigger_sweep and self.async_callback:
-                self._perform_full_sweep()
-            return
 
         if self._ss_index is None:
             self.past_end_bytes_discarded += packet_bytes
             return # more data than we asked for
 
         fft_start_time = time.time()
-        # replace with trigger data when available
-        packet = self._trigger_data.get(freq, packet)
         pow_data = compute_fft(self.real_device, packet, self._vrt_context)
         # collect and compute bins
         collect_start_time = time.time()
         ss = self.plan[self._ss_index]
-        take = min(ss.bins_run, ss.bins_keep - self._ss_received)
-        self.bins.extend(pow_data[ss.bins_skip:ss.bins_skip + take])
+        pass_now = 0 if self._ss_received else ss.bins_pass
+        take = min(ss.bins_run - pass_now, ss.bins_keep - self._ss_received)
+        start = ss.bins_skip + pass_now
+        self.bins.extend(pow_data[start:start + take])
         self._ss_received += take
         collect_stop_time = time.time()
 
@@ -327,7 +251,6 @@ class SweepDevice(object):
         # XXX: in case sweep_iterations() does not work
         if not self.continuous:
             self._ss_index = None
-            self._trigger_data = {}
             self.real_device.abort()
             self.real_device.flush()
 
@@ -343,7 +266,7 @@ class SweepDevice(object):
 
 
 
-def plan_sweep(device, fstart, fstop, rbw, min_points=128, max_points=8192):
+def plan_sweep(device, fstart, fstop, rbw, mode, min_points=128, max_points=8192):
     """
     :param device: a device class or instance such as
                    :class:`pyrf.devices.thinkrf.WSA`
@@ -353,6 +276,8 @@ def plan_sweep(device, fstart, fstop, rbw, min_points=128, max_points=8192):
     :type fstop: float
     :param rbw: requested RBW in Hz (output RBW may be smaller than requested)
     :type rbw: float
+    :param mode: sweep mode, 'ZIF left band' or 'ZIF'
+    :type mode: string
     :param min_points: smallest number of points per capture
     :type min_points: int
     :param max_points: largest number of points per capture (due to
@@ -380,6 +305,8 @@ def plan_sweep(device, fstart, fstop, rbw, min_points=128, max_points=8192):
     device.properties.DC_OFFSET_BW
       the range of frequencies around center that may be affected by
       a DC offset and should not be used
+    device.properties.TUNING_RESOLUTION
+      the smallest tuning increment for fcenter and fstep
 
     :returns: (actual fstart, actual fstop, list of SweepStep instances)
 
@@ -391,95 +318,122 @@ def plan_sweep(device, fstart, fstop, rbw, min_points=128, max_points=8192):
     3. bins[bins_skip:bins_skip + bins_run] are selected
     4. take logarithm of output bins and appended to the result
     5. for sweeps repeat from 2 until the sweep is complete
-    6. bins_keep is the total number of selected bins to keep; for
+    6. bins_pass is the number of selected bins to skip from the first
+       capture only
+    7. bins_keep is the total number of selected bins to keep; for
        single captures bins_run == bins_keep
     """
+    assert mode in ('ZIF left band', 'ZIF')
+    rfe_mode = 'ZIF'
     prop = device.properties
     out = []
-    usable2 = prop.USABLE_BW / 2.0
+    usable2 = prop.USABLE_BW[rfe_mode] / 2.0
     dc_offset2 = prop.DC_OFFSET_BW / 2.0
+    full_bw = prop.FULL_BW[rfe_mode]
 
-    # FIXME: truncate to left-hand sweep area for now
-    fstart = max(prop.MIN_TUNABLE - usable2, fstart)
-    fstop = min(prop.MAX_TUNABLE - dc_offset2, fstop)
+    fstart = max(prop.MIN_TUNABLE[rfe_mode] - usable2, fstart)
+    fstop = min(prop.MAX_TUNABLE[rfe_mode] + (
+        usable2 if mode == 'ZIF' else -dc_offset2), fstop)
 
     if fstop <= fstart:
         return (fstart, fstart, [])
 
-    points = prop.FULL_BW / rbw
-    points = max(min_points, 2 ** math.ceil(math.log(points, 2)))
+    points = full_bw / rbw
+    points = int(max(min_points, 2 ** math.ceil(math.log(points, 2))))
 
     decimation = 1
     ideal_decimation = 2 ** math.ceil(math.log(float(points) / max_points, 2))
-    min_decimation = max(2, prop.MIN_DECIMATION)
-    max_decimation = 2 ** math.floor(math.log(prop.MAX_DECIMATION, 2))
-    if max_points < points and min_decimation <= ideal_decimation:
+    min_decimation = max(2, prop.MIN_DECIMATION[rfe_mode])
+    max_decimation = 2 ** math.floor(math.log(prop.MAX_DECIMATION[rfe_mode], 2))
+    if points > max_points and ideal_decimation >= min_decimation:
+        # decimate because number of points required for rbw is too large
         decimation = min(max_decimation, ideal_decimation)
         points /= decimation
-        decimated_bw = prop.FULL_BW / decimation
+        decimated_bw = full_bw / decimation
         decimation_edge_bins = math.ceil(points * prop.DECIMATED_USABLE / 2.0)
         decimation_edge = decimation_edge_bins * decimated_bw / points
 
-    bin_size = prop.FULL_BW / decimation / float(points)
+    bin_size = float(full_bw) / decimation / points
 
-    # there are three regions that need to be handled differently
-    # region 0: direct digitization / "VLOW band"
-    if fstart < prop.MIN_TUNABLE - usable2:
-        raise NotImplemented # yet
+    # left-hand sweep area
+    if decimation == 1:
+        left_edge = full_bw / 2.0 - usable2
+        left_bin = math.ceil(left_edge / bin_size)
+        fshift = 0 # always preferred
+        wasted_left = left_bin * bin_size - left_edge
+        if mode == 'ZIF':
+            usable_bins = (usable2 - wasted_left) // bin_size * 2
+        else:  # 'ZIF left band'
+            usable_bins = (usable2 - dc_offset2 - wasted_left) // bin_size
 
-    # region 1: left-hand sweep area
-    if prop.MIN_TUNABLE - usable2 <= fstart:
-        if decimation == 1:
-            left_edge = prop.FULL_BW / 2.0 - usable2
-            left_bin = math.ceil(left_edge / bin_size)
-            fshift = left_bin * bin_size - left_edge
-            usable_bins = (usable2 - dc_offset2 - fshift) // bin_size
-        else:
-            left_bin = decimation_edge_bins
-            fshift = usable2 + decimation_edge - (decimated_bw / 2.0)
+    else:
+        left_bin = decimation_edge_bins
+        fshift = usable2 + decimation_edge - (decimated_bw / 2.0)
+        wasted_left = 0 # FIXME
+        if mode == 'ZIF':
+            usable_bins = min(points - (decimation_edge_bins * 2),
+                usable2 // bin_size * 2)
+        else:  # 'ZIF left band'
             usable_bins = min(points - (decimation_edge_bins * 2),
                 (usable2 - dc_offset2) // bin_size)
 
-        usable_bw = usable_bins * bin_size
+    # step_size is limited by tuning resolution. usable_bw is limited by
+    # bin_size. They won't be exactly equal, but try our best
+    step_size = max(1, (usable_bins * bin_size // prop.TUNING_RESOLUTION)
+        ) * prop.TUNING_RESOLUTION
+    # reduce usable_bins to match tuning resolution increment
+    usable_bins = int(max(1, min(usable_bins, round(step_size / bin_size))))
+    usable_bw = usable_bins * bin_size
 
-        fcenter = fstart + usable2
-        # FIXME: fstop not being updated here
-        max_steps = math.floor((prop.MAX_TUNABLE - fstart) / usable_bw)
-        bins_keep = min(round((fstop - fstart) / bin_size),
-            max_steps * usable_bins)
-        sweep_steps = math.ceil(bins_keep / usable_bins)
-        out.append(SweepStep(
-            fcenter=fcenter,
-            fstep=usable_bw,
-            fshift=fshift,
-            decimation=decimation,
-            points=int(points),
-            bins_skip=int(left_bin),
-            bins_run=int(min(usable_bins, bins_keep)),
-            bins_keep=int(bins_keep),
-            ))
+    # start at the next tuning resolution increment left of ideal start
+    fcenter = math.floor((fstart + usable2 - wasted_left)
+        / prop.TUNING_RESOLUTION) * prop.TUNING_RESOLUTION
+    fcenter = max(fcenter, prop.MIN_TUNABLE[rfe_mode])
+    bins_pass = int(round((fstart - (fcenter - usable2 + wasted_left))
+        / bin_size))
+    # we now have our actual fstart
+    fstart = fcenter - bin_size * (points / 2 - left_bin - bins_pass) - fshift
 
-    # region 2: right-hand edge
-    if prop.MAX_TUNABLE - dc_offset2 < fstop:
-        raise NotImplemented # yet
+    # calculate steps and bins
+    step_limit = (prop.MAX_TUNABLE[rfe_mode] - fcenter) // step_size
+    if mode == 'ZIF':
+        right0 = fcenter + usable2 - wasted_left
+    else:  # 'ZIF left band'
+        right_edge = usable2 - usable_bw - wasted_left
+        right0 = fcenter - right_edge
+    steps = 1 + math.ceil((float(fstop) - right0) / step_size)
+    if steps <= step_limit:
+        right_bins = round(usable_bins * ((float(fstop) - right0) %
+            step_size / step_size))
+        if not right_bins:
+            right_bins = usable_bins
+        bins_keep = usable_bins * (steps - 1) - bins_pass + right_bins
+    else:
+        steps = step_limit
+        bins_keep = usable_bins * steps - bins_pass
+
+    # we now have our actual fstop
+    fstop = ((bins_keep + bins_pass - 1) // usable_bins) * step_size + (
+        (bins_keep + bins_pass - 1) % usable_bins + 1) * bin_size + fstart
+
+    assert fcenter % prop.TUNING_RESOLUTION == 0, fcenter
+    assert step_size > 0 and step_size % prop.TUNING_RESOLUTION == 0, step_size
+    assert decimation > 0 and int(decimation) == decimation, decimation
+    assert points > 0 and int(points) == points, points
+    assert left_bin > 0 and int(left_bin) == left_bin, left_bin
+    assert usable_bins > 0 and int(usable_bins) == usable_bins, usable_bins
+    out.append(SweepStep(
+        fcenter=fcenter,
+        fstep=step_size,
+        fshift=fshift,
+        decimation=int(decimation),
+        points=int(points),
+        bins_skip=int(left_bin),
+        bins_run=int(usable_bins),
+        bins_pass=int(bins_pass),
+        bins_keep=int(bins_keep),
+        ))
 
     return (fstart, fstop, out)
 
 
-def trim_sweep_plan(device, plan, fstart, fstop):
-    """
-    :param device: a device class or instance such as
-                   :class:`pyrf.devices.thinkrf.WSA`
-    :param plan: list of :class:`SweepStep` instances
-    :param fstart: starting frequency in Hz
-    :type fstart: float
-    :param fstop: ending frequency in Hz
-    :type fstop: float
-
-    produce a new sweep plan consisting of captures from the passed
-    sweep plan that overlap with the range fstart to fstop.
-    """
-
-    trimmed = (ss.trim(device, fstart, fstop) for ss in plan)
-    return [t for t in trimmed if t]
-
