@@ -40,6 +40,8 @@ def vrt_packet_reader(raw_read):
     raw_read function and accepts the value sent as its data.
     """
     tmpstr = yield raw_read(4)
+    if not tmpstr:
+        return
     (word,) = struct.unpack(">I", tmpstr)
     packet_type = (word >> 28) & 0x0f
     count = (word >> 16) & 0x0f
@@ -348,6 +350,7 @@ class DataPacket(object):
 
         self.valid_data = bool((trailer >> 18) & 1)
         self.reference_lock = bool((trailer >> 17) & 1)
+        self.spec_inv = bool((trailer >> 14) & 1)
         self.over_range = bool((trailer >> 13) & 1)
         self.sample_loss = bool((trailer >> 12) & 1)
 
